/**************************************************************************
*  Copyright (c) 2016 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  27.12.2016  mifi  First Version.
**************************************************************************/
#if !defined(__C5SOC_H__)
#define __C5SOC_H__


/*************************************************************************/
/*    Some global defines                                                */    
/*************************************************************************/

/*
 * Cortex-A9 Core clock frequency in hz
 */
#ifndef MPU_CLK_FREQ
#define MPU_CLK_FREQ       800000000
#endif

/*
 * Cortex-A9 Core Peripheral clock frequency in hz
 */
#ifndef MPU_PERIPH_CLK_FREQ 
#define MPU_PERIPH_CLK_FREQ   200000000
#endif

#ifndef L3_MP_CLK_FREQ
#define L3_MP_CLK_FREQ     200000000
#endif

#ifndef L3_SP_CLK_FREQ
#define L3_SP_CLK_FREQ     100000000
#endif

#ifndef L4_MP_CLK_FREQ
#define L4_MP_CLK_FREQ     100000000
#endif

#ifndef L4_SP_CLK_FREQ
#define L4_SP_CLK_FREQ     100000000
#endif


/*************************************************************************/
/*    Configuration of the Cyclone V SoC Processor and Core Peripherals  */    
/*************************************************************************/

#define __MPU_PRESENT      1
#define __GIC_PRIO_BITS    5
#define __GIC_PRIO_MASK    ((1 << __GIC_PRIO_BITS) - 1)
#define __GIC_PRIO_SHIFT   (8 - __GIC_PRIO_BITS)


/*************************************************************************/
/*    Interrupt Number Definition                                        */    
/*************************************************************************/
typedef enum IRQn
{
/******  Cortex-A9 Processor Exceptions Numbers  *************************/
   /*
    * For more information take a look at:
    *
    *    ARM Cortex-A9 MPCore
    *    Technical Reference Manual 
    */
   SGI0_IRQn            = 0,     /* Software Generated Interrupts (SGI) */
   SGI1_IRQn            = 1,
   SGI2_IRQn            = 2,
   SGI3_IRQn            = 3,
   SGI4_IRQn            = 4,
   SGI5_IRQn            = 5,
   SGI6_IRQn            = 6,
   SGI7_IRQn            = 7,
   SGI8_IRQn            = 8,
   SGI9_IRQn            = 9,
   SGI10_IRQn           = 10,
   SGI11_IRQn           = 11,
   SGI12_IRQn           = 12,
   SGI13_IRQn           = 13,
   SGI14_IRQn           = 14,
   SGI15_IRQn           = 15,
   GlobalTimer_IRQn     = 27,    /* Global timer */
   FIQPin_IRQn          = 28,    /* A legacy nFIQ pin */
   PrivateTimer_IRQn    = 29,
   WatchdogTimer_IRQn   = 30,    /* Watchdog timer */
   IRQPin_IRQn          = 31,    /* A legacy nIRQ pin */

/******  Cyclone V SoC specific Interrupt Numbers  ***********************/
   /*
    * For more information take a look at:
    *
    *    Cyclone V Hard Processor System
    *    Technical Reference Manual
    */
   FPGA_IRQ0_IRQn       = 72,
   FPGA_IRQ63_IRQn      = 135,
   
   SPI0_IRQn            = 186,
   SPI1_IRQn            = 187,
   SPI2_IRQn            = 188,
   SPI3_IRQn            = 189,
   
   I2C0_IRQn            = 190,
   I2C1_IRQn            = 191,
   I2C2_IRQn            = 192,
   I2C3_IRQn            = 193,
   
   UART0_IRQn           = 194,
   UART1_IRQn           = 195,
   
   GPIO0_IRQn           = 196,
   GPIO1_IRQn           = 197,
   GPIO2_IRQn           = 198,
   
   Timer0_IRQn          = 199,
   Timer1_IRQn          = 200,
   Timer2_IRQn          = 201,
   Timer3_IRQn          = 202,
   
/*************************************************************************/
   MAX_IRQ_COUNT        = 256       
} IRQn_Type;


/*************************************************************************/
/*    Device Specific Peripheral Section                                 */
/*************************************************************************/

#define __IO   volatile

/*
 * Global Interrupt Controller: Interrupt Interface
 */
typedef struct
{
   __IO uint32_t ICCICR;            /* CPU Interface Control Register */
   __IO uint32_t ICCPMR;            /* Interrupt Priority Mask Register */
   __IO uint32_t ICCBPR;            /* Binary Point Register */
   __IO uint32_t ICCIAR;            /* Interrupt Acknowledge Register */
   __IO uint32_t ICCEOIR;           /* End Of Interrupt Register */
   __IO uint32_t ICCRPR;            /* Running Priority Register */
   __IO uint32_t ICCHPIR;           /* Highest Pending Interrupt Register */
   __IO uint32_t ICCABPR;           /* Aliased Non-secure Binary Point Register */
   __IO uint32_t Reserve[55];       /* Reserved */
   __IO uint32_t ICCIDR;            /* CPU Interface Implementer Identification Register */
} GIC_TypeDef;


/*
 * Global Interrupt Controller: Distributor
 */
typedef struct
{
   __IO uint32_t ICDDCR;            /* Distributor Control Register */
   __IO uint32_t ICDICTR;           /* Interrupt Controller Type Register */
   __IO uint32_t ICDIIDR;           /* Distributor Implementer Identification Register */
   __IO uint32_t Reserve0[29];
   __IO uint32_t ICDISR[8];         /* Interrupt Security Registers */
   __IO uint32_t Reserve1[24];
   __IO uint32_t ICDISER[8];        /* Interrupt Set-Enable Registers */
   __IO uint32_t Reserve2[24];
   __IO uint32_t ICDICER[8];        /* Interrupt Clear-Enable Registers */
   __IO uint32_t Reserve3[24];
   __IO uint32_t ICDISPR[8];        /* Interrupt Set-Pending Registers */
   __IO uint32_t Reserve4[24];
   __IO uint32_t ICDICPR[8];        /* Interrupt Clear-Pending Registers */
   __IO uint32_t Reserve5[24];
   __IO uint32_t ICDABR[8];         /* Active Bit Registers */
   __IO uint32_t Reserve6[56];
   __IO uint32_t ICDIPR[64];        /* Interrupt Priority Registers */
   __IO uint32_t Reserve7[192];
   __IO uint32_t ICDIPTR[64];       /* Interrupt Processor Targets Registers */
   __IO uint32_t Reserve8[192];
   __IO uint32_t ICDICFR[16];       /* Interrupt Configuration Registers */
   __IO uint32_t Reserve9[48];
   __IO uint32_t ICPPISR;           /* PPI Status Register */
   __IO uint32_t ICSPISR[7];        /* SPI Status Registers */
   __IO uint32_t Reserve10[120];
   __IO uint32_t ICDSGIR;           /* Software Generated Interrupt Register */
} GICDist_TypeDef;


/*
 * Global Timer
 */
typedef struct
{
   __IO uint32_t CounterLow;        /* Counter Low Register */
   __IO uint32_t CounterHigh;       /* Counter High Register */
   __IO uint32_t Control;           /* Control Register */
   __IO uint32_t InterruptStatus;   /* Interrupt Status Register*/
   __IO uint32_t ValueLow;          /* Comparator Value Low Register */
   __IO uint32_t ValueHigh;         /* Comparator Value High Register */
   __IO uint32_t AutoIncrement;     /* Auto-increment Register*/
} GlobalTimer_TypeDef; 


/*
 * Private Timer
 */
typedef struct
{
   __IO uint32_t Load;              /* Load Register */
   __IO uint32_t Counter;           /* Counter Register */
   __IO uint32_t Control;           /* Control Register */
   __IO uint32_t InterruptStatus;   /* Interrupt Status Register */
} PrivateTimer_TypeDef;


/*
 * Watchdog Timer
 */
typedef struct
{
   __IO uint32_t Load;              /* Load Register */
   __IO uint32_t Counter;           /* Counter Register */
   __IO uint32_t Control;           /* Control Register */
   __IO uint32_t InterruptStatus;   /* Interrupt Status Register */
   __IO uint32_t ResetStatus;       /* Reset Status Register */
   __IO uint32_t Disable;           /* Disable Register */
} WatchdogTimer_TypeDef;


/*
 * GPIO
 */
typedef struct
{
   __IO uint32_t DR;                /* Data Register */ 
   __IO uint32_t DDR;               /* Data Direction Register */
   __IO uint32_t Reserve0[10];
   __IO uint32_t INT_EN;            /* Interrupt Enable Register */
   __IO uint32_t INT_MASK;          /* Interrupt Mask Register */
   __IO uint32_t INT_TYPE_LEVEL;    /* Interrupt Level Register */
   __IO uint32_t INT_POLARITY;      /* Interrupt Polarity Register */
   __IO uint32_t INT_STATUS;        /* Interrupt Status Register */
   __IO uint32_t RAW_INT_STATUS;    /* Raw Interrupt Status Register */
   __IO uint32_t DEBOUNCE;          /* Debounce Enable Register */
   __IO uint32_t PORTA_EOI;         /* Clear Interrupt Register */
   __IO uint32_t EXT_PORTA;         /* External Port A Register */ 
   __IO uint32_t Reserve1[3];
   __IO uint32_t LS_SYNC;           /* Synchronization Level Register */
   __IO uint32_t ID_CODE;           /* ID Code Register */
   __IO uint32_t Reserve2; 
   __IO uint32_t VER_ID_CODE;       /* GPIO Version Register */
   __IO uint32_t CONFIG_REG2;       /* Configuration Register 2 */
   __IO uint32_t CONFIG_REG1;       /* Configuration Register 1 */
} GPIO_TypeDef;


/*
 * UART
 */
typedef struct
{
   union {
      __IO uint32_t RBR;            /* Rx Buffer */
      __IO uint32_t THR;            /* Tx Holding */
      __IO uint32_t DLL;            /* Divisor Latch Low */
   };
   
   union {
      __IO uint32_t IER;            /* Interrupt Enable */
      __IO uint32_t DLH;            /* Divisor Latch High */
   };
   
   union {
      __IO uint32_t IIR;            /* Interrupt Identity Register (when read) */
      __IO uint32_t FCR;            /* FIFO Control Register (when written) */
   };
   
   __IO uint32_t LCR;               /* Line Control Register */
   __IO uint32_t MCR;               /* Modem Control Register */
   __IO uint32_t LSR;               /* Line Status Register */
   __IO uint32_t MSR;               /* Modem Status Register */
   __IO uint32_t SCR;               /* Scratchpad Register */
   __IO uint32_t Reserve0[4];
   __IO uint32_t SRBR;              /* Shadow Receive Buffer Register */
   __IO uint32_t STHR;              /* Shadow Transmit Buffer Register */
   __IO uint32_t Reserve1[14];
   __IO uint32_t FAR;               /*FIFO Access Register */
   __IO uint32_t TFR;               /* Transmit FIFO Read Register */
   __IO uint32_t RFW;               /* Receive FIFO Write */
   __IO uint32_t USR;               /* UART Status Register */
   __IO uint32_t TFL;               /* Transmit FIFO Level */
   __IO uint32_t RFL;               /* Receive FIFO LEvel */
   __IO uint32_t SRR;               /* Software Reset Register */
   __IO uint32_t SRTS;              /* Shadow Request to Send */
   __IO uint32_t SBCR;              /* Shadow Break Control Register */
   __IO uint32_t SDMAM;             /* Shadow DMA Mode */
   __IO uint32_t SFE;               /* Shadow FIFO Enable */
   __IO uint32_t SRT;               /* Shadow Rx Trigger */
   __IO uint32_t STET;              /* Shadow Tx Empty Trigger */
   __IO uint32_t HTX;               /* Halt Tx */
   __IO uint32_t DMASA;             /* DMA Software Acknowledge */
   __IO uint32_t Reserve2[18];
   __IO uint32_t CPR;               /* Component Parameter Register */
   __IO uint32_t UCV;               /* Component Version */
   __IO uint32_t CTR;               /* Component Type Register */
} UART_TypeDef;


/*
 * Timer
 */
typedef struct
{
   __IO uint32_t LoadCount;         /* Load Count Register */
   __IO uint32_t CurrentValue;      /* Current Value Register */
   __IO uint32_t Control;           /* Control Register */   
   __IO uint32_t EOI;               /* End-of-Interrupt Register */
   __IO uint32_t InterruptStatus;   /* Interrupt Status Register */   
   __IO uint32_t Reserve0[38];
   __IO uint32_t ComponentVersion;  /* Component Version Register */
} TIMER_TypeDef;

 
/*************************************************************************/
/*    Peripheral memory map                                              */
/*************************************************************************/
 
#define SCU_BASE              0xFFFEC000
#define GIC_BASE              0xFFFEC100
#define GLOBAL_TIMER_BASE     0xFFFEC200
#define PRIVATE_TIMER_BASE    0xFFFEC600
#define WATCHDOG_TIMER_BASE   0xFFFEC620
#define GIC_DIST_BASE         0xFFFED000

#define GPIO0_BASE            0xFF708000
#define GPIO1_BASE            0xFF709000
#define GPIO2_BASE            0xFF70A000

#define UART0_BASE            0xFFC02000
#define UART1_BASE            0xFFC03000

#define SP_TIMER0_BASE        0xFFC08000
#define SP_TIMER1_BASE        0xFFC09000
#define OSC1_TIMER0_BASE      0xFFD00000
#define OSC1_TIMER1_BASE      0xFFD01000


/*************************************************************************/
/*    Peripheral declaration                                             */
/*************************************************************************/

#define GIC                   ((GIC_TypeDef*)            GIC_BASE)
#define GIC_DIST              ((GICDist_TypeDef*)        GIC_DIST_BASE)
#define GLOBAL_TIMER          ((GlobalTimer_TypeDef *)   GLOBAL_TIMER_BASE)
#define PRIVATE_TIMER         ((PrivateTimer_TypeDef *)  PRIVATE_TIMER_BASE)
#define WATCHDOG_TIMER        ((WatchdogTimer_TypeDef *) WATCHDOG_TIMER_BASE)

#define GPIO0                 ((GPIO_TypeDef *)          GPIO0_BASE) 
#define GPIO1                 ((GPIO_TypeDef *)          GPIO1_BASE) 
#define GPIO2                 ((GPIO_TypeDef *)          GPIO2_BASE) 

#define UART0                 ((UART_TypeDef *)          UART0_BASE)
#define UART1                 ((UART_TypeDef *)          UART1_BASE)

#define TIMER0                ((TIMER_TypeDef *)         SP_TIMER0_BASE)
#define TIMER1                ((TIMER_TypeDef *)         SP_TIMER1_BASE)
#define TIMER2                ((TIMER_TypeDef *)         OSC1_TIMER0_BASE)
#define TIMER3                ((TIMER_TypeDef *)         OSC1_TIMER1_BASE)
#define SP_TIMER0             ((TIMER_TypeDef *)         SP_TIMER0_BASE)
#define SP_TIMER1             ((TIMER_TypeDef *)         SP_TIMER1_BASE)
#define OSC1_TIMER0           ((TIMER_TypeDef *)         OSC1_TIMER0_BASE)
#define OSC1_TIMER1           ((TIMER_TypeDef *)         OSC1_TIMER1_BASE)
 
#endif /* !__C5SOC_H__ */
/*** EOF ***/
